@@ -2,14 +2,15 @@
 # @Author: Kelvin
 # @Date:   2021-02-06 13:18:58
 # @Last Modified by:   Kelvin
-# @Last Modified time: 2021-02-16 15:59:09
+# @Last Modified time: 2021-02-20 09:07:55
+from typing import Union, Sequence, Tuple
 
 modules = ['dandelion', 'pandas', 'numpy', 'matplotlib',
            'networkx', 'scipy', 'skbio', 'distance', 'polyleven']
 
 
 # borrowed from scanpy's logging module
-def _versions_dependencies(dependencies):
+def _versions_dependencies(dependencies: Sequence):
     for mod in dependencies:
         mod_name, dist_name = mod if isinstance(mod, tuple) else (mod, mod)
         try:
@@ -19,7 +20,7 @@ def _versions_dependencies(dependencies):
             pass
 
 
-def print_versions(dependencies=modules):
+def print_versions(dependencies: Sequence = modules):
     '''
     Versions that are essential for dandelion's operation.
     '''
@@ -28,7 +29,8 @@ def print_versions(dependencies=modules):
         for mod, ver in _versions_dependencies(dependencies)
     ))
 
-def print_header(dependencies=modules):
+
+def print_header(dependencies: Sequence = modules):
     '''
     Versions that are essential for dandelion's operation.
     '''
