@@ -15,13 +15,21 @@ def test_write_airr(create_testfolder, airr_10x):
     assert len(list(create_testfolder.iterdir())) == 1
 
 
-def test_read10xairr(create_testfolder):
+def test_readwrite10xairr(create_testfolder):
     airr_file = str(create_testfolder) + "/test_airr_rearrangements.tsv"
+    airr_file2 = str(create_testfolder) + "/test_airr_rearrangements2.tsv"
     vdj = ddl.read_10x_airr(airr_file)
     assert vdj.data.shape[0] == 9
     assert vdj.data.shape[1] == 32
     assert vdj.metadata.shape[0] == 5
-    assert vdj.metadata.shape[1] == 25
+    assert vdj.metadata.shape[1] == 26
+    vdj.write_airr(airr_file2)
+    vdj2 = ddl.read_10x_airr(airr_file2)
+    assert vdj2.data.shape[0] == 9
+    assert vdj2.data.shape[1] == 32
+    assert vdj2.metadata.shape[0] == 5
+    assert vdj2.metadata.shape[1] == 26
+    os.remove(airr_file2)
 
 
 def test_read10xvdj_json(create_testfolder, json_10x_cr6):
@@ -32,7 +40,7 @@ def test_read10xvdj_json(create_testfolder, json_10x_cr6):
     assert vdj.data.shape[0] == 26
     assert vdj.data.shape[1] == 49
     assert vdj.metadata.shape[0] == 10
-    assert vdj.metadata.shape[1] == 29
+    assert vdj.metadata.shape[1] == 30
     os.remove(json_file)
 
 
@@ -43,16 +51,16 @@ def test_read10xvdj_cr6(create_testfolder, json_10x_cr6, annotation_10x_cr6, fas
     annotation_10x_cr6.to_csv(annot_file, index=False)
     vdj = ddl.read_10x_vdj(annot_file)
     assert vdj.data.shape[0] == 26
-    assert vdj.data.shape[1] == 33
+    assert vdj.data.shape[1] == 31
     assert vdj.metadata.shape[0] == 10
-    assert vdj.metadata.shape[1] == 29
+    assert vdj.metadata.shape[1] == 30
     with open(json_file, 'w') as outfile:
         json.dump(json_10x_cr6, outfile)
     vdj = ddl.read_10x_vdj(annot_file)
     assert vdj.data.shape[0] == 26
     assert vdj.data.shape[1] == 49
     assert vdj.metadata.shape[0] == 10
-    assert vdj.metadata.shape[1] == 29
+    assert vdj.metadata.shape[1] == 30
     assert not vdj.data.sequence.empty
     os.remove(json_file)
     fh = open(fasta_file, "w")
@@ -63,9 +71,9 @@ def test_read10xvdj_cr6(create_testfolder, json_10x_cr6, annotation_10x_cr6, fas
         ddl.utl.Write_output(out, fasta_file)
     vdj = ddl.read_10x_vdj(annot_file)
     assert vdj.data.shape[0] == 26
-    assert vdj.data.shape[1] == 34
+    assert vdj.data.shape[1] == 32
     assert vdj.metadata.shape[0] == 10
-    assert vdj.metadata.shape[1] == 29
+    assert vdj.metadata.shape[1] == 30
     assert not vdj.data.sequence.empty
     os.remove(fasta_file)
 
@@ -76,9 +84,9 @@ def test_read10xvdj(create_testfolder, annotation_10x, fasta_10x):
     annotation_10x.to_csv(annot_file, index=False)
     vdj = ddl.read_10x_vdj(annot_file)
     assert vdj.data.shape[0] == 9
-    assert vdj.data.shape[1] == 20
+    assert vdj.data.shape[1] == 18
     assert vdj.metadata.shape[0] == 5
-    assert vdj.metadata.shape[1] == 27
+    assert vdj.metadata.shape[1] == 28
     fh = open(fasta_file, "w")
     fh.close()
     out = ''
@@ -87,9 +95,9 @@ def test_read10xvdj(create_testfolder, annotation_10x, fasta_10x):
         ddl.utl.Write_output(out, fasta_file)
     vdj = ddl.read_10x_vdj(annot_file)
     assert vdj.data.shape[0] == 9
-    assert vdj.data.shape[1] == 21
+    assert vdj.data.shape[1] == 19
     assert vdj.metadata.shape[0] == 5
-    assert vdj.metadata.shape[1] == 27
+    assert vdj.metadata.shape[1] == 28
     assert not vdj.data.sequence.empty
     os.remove(fasta_file)
 
@@ -101,16 +109,16 @@ def test_read10xvdj_cr6_folder(create_testfolder, json_10x_cr6, annotation_10x_c
     annotation_10x_cr6.to_csv(annot_file, index=False)
     vdj = ddl.read_10x_vdj(str(create_testfolder))
     assert vdj.data.shape[0] == 26
-    assert vdj.data.shape[1] == 33
+    assert vdj.data.shape[1] == 31
     assert vdj.metadata.shape[0] == 10
-    assert vdj.metadata.shape[1] == 29
+    assert vdj.metadata.shape[1] == 30
     with open(json_file, 'w') as outfile:
         json.dump(json_10x_cr6, outfile)
     vdj = ddl.read_10x_vdj(str(create_testfolder))
     assert vdj.data.shape[0] == 26
     assert vdj.data.shape[1] == 49
     assert vdj.metadata.shape[0] == 10
-    assert vdj.metadata.shape[1] == 29
+    assert vdj.metadata.shape[1] == 30
     assert not vdj.data.sequence.empty
     os.remove(json_file)
     fh = open(fasta_file, "w")
@@ -121,9 +129,9 @@ def test_read10xvdj_cr6_folder(create_testfolder, json_10x_cr6, annotation_10x_c
         ddl.utl.Write_output(out, fasta_file)
     vdj = ddl.read_10x_vdj(str(create_testfolder))
     assert vdj.data.shape[0] == 26
-    assert vdj.data.shape[1] == 34
+    assert vdj.data.shape[1] == 32
     assert vdj.metadata.shape[0] == 10
-    assert vdj.metadata.shape[1] == 29
+    assert vdj.metadata.shape[1] == 30
     assert not vdj.data.sequence.empty
     os.remove(fasta_file)
 
@@ -134,9 +142,9 @@ def test_read10xvdj_folder(create_testfolder, annotation_10x, fasta_10x):
     annotation_10x.to_csv(annot_file, index=False)
     vdj = ddl.read_10x_vdj(str(create_testfolder))
     assert vdj.data.shape[0] == 9
-    assert vdj.data.shape[1] == 20
+    assert vdj.data.shape[1] == 18
     assert vdj.metadata.shape[0] == 5
-    assert vdj.metadata.shape[1] == 27
+    assert vdj.metadata.shape[1] == 28
     fh = open(fasta_file, "w")
     fh.close()
     out = ''
@@ -145,9 +153,9 @@ def test_read10xvdj_folder(create_testfolder, annotation_10x, fasta_10x):
         ddl.utl.Write_output(out, fasta_file)
     vdj = ddl.read_10x_vdj(str(create_testfolder))
     assert vdj.data.shape[0] == 9
-    assert vdj.data.shape[1] == 21
+    assert vdj.data.shape[1] == 19
     assert vdj.metadata.shape[0] == 5
-    assert vdj.metadata.shape[1] == 27
+    assert vdj.metadata.shape[1] == 28
     assert not vdj.data.sequence.empty
     os.remove(fasta_file)
 
@@ -158,9 +166,9 @@ def test_to_scirpy(create_testfolder, annotation_10x, fasta_10x):
     annotation_10x.to_csv(annot_file, index=False)
     vdj = ddl.read_10x_vdj(str(create_testfolder))
     assert vdj.data.shape[0] == 9
-    assert vdj.data.shape[1] == 20
+    assert vdj.data.shape[1] == 18
     assert vdj.metadata.shape[0] == 5
-    assert vdj.metadata.shape[1] == 27
+    assert vdj.metadata.shape[1] == 28
     adata = ddl.to_scirpy(vdj)
     assert adata.obs.shape[0] == 5
     assert adata.obs.shape[1] == 43
@@ -172,9 +180,9 @@ def test_to_scirpy(create_testfolder, annotation_10x, fasta_10x):
         ddl.utl.Write_output(out, fasta_file)
     vdj = ddl.read_10x_vdj(str(create_testfolder))
     assert vdj.data.shape[0] == 9
-    assert vdj.data.shape[1] == 21
+    assert vdj.data.shape[1] == 19
     assert vdj.metadata.shape[0] == 5
-    assert vdj.metadata.shape[1] == 27
+    assert vdj.metadata.shape[1] == 28
     assert not vdj.data.sequence.empty
     adata = ddl.to_scirpy(vdj)
     assert adata.obs.shape[0] == 5
@@ -192,9 +200,9 @@ def test_tofro_scirpy_cr6(create_testfolder, annotation_10x_cr6, json_10x_cr6):
         json.dump(json_10x_cr6, outfile)
     vdj = ddl.read_10x_vdj(str(create_testfolder), filename_prefix = 'test_all')
     assert vdj.data.shape[0] == 26
-    assert vdj.data.shape[1] == 33
+    assert vdj.data.shape[1] == 31
     assert vdj.metadata.shape[0] == 10
-    assert vdj.metadata.shape[1] == 29
+    assert vdj.metadata.shape[1] == 30
     adata = ddl.to_scirpy(vdj)
     assert adata.obs.shape[0] == 10
     assert adata.obs.shape[1] == 43
